# encoding: utf-8
# @Author: FAULHABERmotioncontrol
# @Date:   2021-06-25T07:08:57+02:00
# @Email:  FAULHABERMotioncontrol@users.noreply.github.com
# @Project: motioncontrollib
# @Filename: gui.py
# @Last modified by:   FAULHABERmotioncontrol
# @Last modified time: 2023-05-25T09:30:22+02:00
# @License: # This module is part of motioncontrollib and is released under
# <see LICENSE file>
# @Copyright: # Copyright (C) 2020-2020 the motioncontrollib authors and contributors
# <see AUTHORS file>


"""
example for using this module in with a gui (tkinter is used here).

you can check out other libraries like kivy or pyqt
"""
import logging
import tkinter
import tkinter.messagebox
import tkinter.scrolledtext

import momanlibpy


class CMomanLibSample:
    """
    This class provides an implementation with the momanlib as a backend for the gui.

    It initializes with `Init`, provides synchronous access to sdo with
    `GetStrObj`, `SetObj` and handles the asynchronous callbacks from the lib
    """

    def __init__(self):
        """simple init function"""
        self.statusword = 0
        self.abortmsg = ""

    def Init(self, callback_AsyncDataReceived):
        """initializes the library with a python callback function"""
        self.lib = momanlibpy.MomanProt(momanlibpy.DLL_PATH_CO_USB)
        interface_result = self.lib.mmProtInitInterface(
            momanlibpy.DLL_PATH_MC3USB,
            callback_AsyncDataReceived,
        )
        if interface_result is momanlibpy.eMomanprot.eMomanprot_ok:
            open_result = self.lib.mmProtOpenCom(
                1,
                0,
                0,
            )  # 1 0 0 for usb1, 2 0 0 for usb2
            if open_result == momanlibpy.eMomanprot.eMomanprot_ok:
                try:
                    self.lib.mmProtGetObj(1, 0x1000, 0x00)
                except BaseException:
                    pass  # dont care for now, handle in your Application as you like
                return True
        return False

    def GetStrObj(self, nodeNr, index, subIndex) -> bytes:
        """returns a bytes-string from the library

        can be used to get the devicename, motorname,...
        returns <ERROR> by default
        """
        value = "<ERROR>"
        try:
            ret, answData = self.lib.mmProtGetStrObj(nodeNr, index, subIndex)

            if ret == momanlibpy.eMomanprot.eMomanprot_ok:
                value = answData
            else:
                # we could raise an exception here
                pass
        except OSError:
            logging.warning("It could be necessary to just restart this program")
            exit(0)
        return value

    def SendCommand(self, nodeNr, command):
        """send command can be used to reset the device, and change the statemachine

        is used here to enable and disable the powerstage
        """
        try:
            return self.lib.mmProtSendCommand(nodeNr, 0x0000, int(command.value), 0, 0)
        except BaseException:
            pass

    def SetObj(self, nodeNr, objIdx, subObjIdx, value, bytesize):
        """SetObj initiates a sdo-download to the device

        can be used to get the devicename, motorname,...
        returns <ERROR> by default
        """
        ret, abortCode = self.lib.mmProtSetObj(
            nodeNr,
            objIdx,
            subObjIdx,
            value,
            bytesize,
        )
        if ret == momanlibpy.eMomanprot.eMomanprot_ok:
            return True
        else:
            m_abortMessage = self.lib.mmProtGetAbortMessage(abortCode)
            self.abortmsg = m_abortMessage
            logging.warning(m_abortMessage)
            return False

    def GetStatusword(self):
        """getter for the statusword"""
        return self.statusword

    def GetAbortMessage(self) -> str:
        """returns an abortMessage"""
        return self.abortmsg

    def ReadReceivedData(self) -> [bool, str, str]:
        """read the data returned in a callback"""
        ret, answData, nodeNr, cmdString, receiveTelegram = self.lib.mmProtReadAnswer()
        if ret != momanlibpy.eMomanprot.eMomanprot_noData:
            data = answData
            cmd = cmdString
            ret, value = self.lib.mmProtDecodeAnswStr(data)
            if ret == momanlibpy.eDecoded.eDecoded_Statusword:
                self.statusword = int(value)
            ret = True
        else:
            data = ""
            cmd = ""
            ret = False
        return ret, data, cmd


class Application(tkinter.Frame):
    """default Tkinter Application"""

    cNodeNr = 1

    def __init__(self, master=None):
        """initializes the tkinter app"""
        super().__init__(master)
        self.master = master
        # setting title
        master.title("FAULHABER DemoPython")
        # setting window size
        width = 634
        height = 300
        screenwidth = master.winfo_screenwidth()
        screenheight = master.winfo_screenheight()
        alignstr = f"{int(width)}x{int(height)}+{int((screenwidth - width) / 2)}+{int((screenheight - height) / 2)}"
        master.geometry(alignstr)
        master.resizable(width=False, height=False)
        statusbar = self.statusbar = tkinter.Label(
            self.master,
            text="",
            bd=1,
            relief=tkinter.SUNKEN,
            anchor=tkinter.W,
        )

        statusbar.pack(side=tkinter.BOTTOM, fill=tkinter.X)
        if master:
            master.protocol("WM_DELETE_WINDOW", self.on_closing)

        self.create_widgets()
        self._entry()

    def on_closing(self):
        try:
            if hasattr(self, "momanLibSample"):
                if self.momanLibSample:
                    if hasattr(self.momanLibSample, "lib"):
                        # as described in EN_7000_05064.pdf 3.1.1 Synchronous communication
                        self.momanLibSample.lib.mmProtCloseCom()
                        self.momanLibSample.lib.mmProtCloseInterface()
        except:
            logging.exception()
        if hasattr(self, "master"):
            self.master.destroy()

    def create_widgets(self):
        """creates the application widgets, like in the tkinter example"""
        self.create_sync_sample()
        self.create_async_sample()
        self.create_pos_sample()
        self.create_commlog()

    def create_sync_sample(self):
        """creates the sync-sample with the buttons"""
        self.lbl_syncsample = tkinter.Label(
            self.master,
            text="Sync. Sample:",
            justify="center",
        )
        self.btn_get_str_object = tkinter.Button(
            self.master,
            text="GetStrObj",
            command=self.click_get_str_obj,
        )

        self.lbl_syncsample.place(x=23, y=31, width=75, height=13)
        self.btn_get_str_object.place(x=116, y=26, width=76, height=23)

    def create_async_sample(self):
        """creates the async-sample with the buttons"""
        self.lbl_asyncsample = tkinter.Label(
            self.master,
            text="Asnc. Sample:",
            justify="center",
        )

        self.btn_shutdown = tkinter.Button(
            self.master,
            text="Shutdown",
            command=self.click_shutdown,
            state="disabled",
        )
        self.btn_switchon = tkinter.Button(
            self.master,
            text="SwitchOn",
            command=self.click_switchon,
            state="disabled",
        )
        self.btn_enop = tkinter.Button(
            self.master,
            text="EnOp",
            command=self.click_enop,
            state="disabled",
        )

        self.lbl_asyncsample.place(x=23, y=62, width=80, height=13)
        self.btn_shutdown.place(x=116, y=57, width=76, height=23)
        self.btn_switchon.place(x=198, y=57, width=76, height=23)
        self.btn_enop.place(x=280, y=57, width=76, height=23)

    def create_pos_sample(self):
        """creates the positionsync -sample with the buttons"""
        self.lbl_asyncsample = tkinter.Label(
            self.master,
            text="Position Sample:",
            justify="center",
        )
        self.btn_pos_sample = tkinter.Button(
            self.master,
            text="Rel. Position",
            command=self.click_relpos,
            state="disabled",
        )

        self.lbl_asyncsample.place(x=13, y=91, width=85, height=13)
        self.btn_pos_sample.place(x=116, y=86, width=76, height=23)

    def create_commlog(self):
        """creates the log, so it is (somewhat) visible what is happening"""
        self.lbl_commlog = tkinter.Label(
            self.master,
            text="Communication log:",
            justify="center",
        )
        self.commlog = tkinter.scrolledtext.ScrolledText(self.master)

        self.lbl_commlog.place(x=0, y=129, width=125, height=16)
        self.commlog.place(x=0, y=152, width=633, height=126)

    def _entry(self):
        """entry method for the instantiation of the CMomanLibSample"""
        self.momanLibSample = CMomanLibSample()
        if self.momanLibSample.Init(self.CBAsyncDataReceived):
            self.statusbar.config(text="FAULHABER communication API loaded")
            self.btn_shutdown.config(state="active")
            self.btn_switchon.config(state="active")
            self.btn_enop.config(state="active")
            self.momanlib_loaded = True
        else:
            self.momanlib_loaded = False
            tkinter.messagebox.showinfo(message="Interface can not be opened!")

    def AddText(self, text):
        """adds text to the log"""
        self.commlog.insert(tkinter.END, f"{text}\r\n")
        self.commlog.see("end")

    def CBAsyncDataReceived(self):
        """is called by the lib to react on asynchronous events"""
        self.after(10, self.RecallCBAsyncDataReceived)

    def RecallCBAsyncDataReceived(self):
        """is called by tkinter to react on asynchronous events in CBAsyncDataReceived"""
        ret, data, cmd = self.momanLibSample.ReadReceivedData()
        if cmd != "":
            self.AddText(
                f'Async Received: {data.decode("utf8")} (Cmd = {cmd.decode("utf8")})',
            )
        else:
            self.AddText(f"Async Received: {data}")

        if (self.momanLibSample.GetStatusword() & 0x00FF) == 0x0027:
            # "Operation Enabled"
            self.btn_pos_sample.config(state="active")
        else:
            self.btn_pos_sample.config(state="disabled")

    def click_get_str_obj(self):
        """click handler for get-string example button"""
        if self.momanlib_loaded:
            value = self.momanLibSample.GetStrObj(self.cNodeNr, 0x1008, 0x00)
            self.AddText("Sync Read 0x1008.00")
            self.AddText("Sync Received: " + str(value))

    def click_shutdown(self):
        """click handler for powerstage shutdown example button"""
        if self.momanlib_loaded:
            if self.momanLibSample.SendCommand(
                self.cNodeNr,
                momanlibpy.eMomancmd.eMomancmd_shutdown,
            ):
                self.AddText("Send SHUTDOWN")

    def click_switchon(self):
        """click handler for powerstage switch on example button"""
        if self.momanlib_loaded:
            if self.momanLibSample.SendCommand(
                self.cNodeNr,
                momanlibpy.eMomancmd.eMomancmd_switchon,
            ):
                self.AddText("Send SWITCHON")

    def click_enop(self):
        """click handler for powerstage enable operation example button"""
        if self.momanlib_loaded:
            if self.momanLibSample.SendCommand(
                self.cNodeNr,
                momanlibpy.eMomancmd.eMomancmd_EnOp,
            ):
                self.AddText("Send ENOP")

    def click_relpos(self):
        """click handler for the relative positioning example button"""
        err = True
        self.AddText("Execute MOVE RELATIVE")
        if self.momanLibSample.SetObj(self.cNodeNr, 0x6060, 0x00, 1, 1):
            if self.momanLibSample.SetObj(self.cNodeNr, 0x607A, 0x00, 1000, 4):
                if self.momanLibSample.SetObj(
                    self.cNodeNr,
                    0x0000,
                    momanlibpy.eMomancmd.eMomancmd_EnOp.value,
                    0,
                    0,
                ):
                    if self.momanLibSample.SetObj(
                        self.cNodeNr,
                        0x6040,
                        0x00,
                        0x007F,
                        2,
                    ):
                        err = False
        if err:
            abortMessage = self.momanLibSample.GetAbortMessage()
            if abortMessage:
                self.AddText("Error Received:" + abortMessage)


def main():
    """main method to start the Tkinter app"""
    root = tkinter.Tk()
    app = Application(master=root)
    app.mainloop()


if __name__ == "__main__":
    main()
